﻿using System.Text;
using Microsoft.Extensions.Logging;

namespace MesETL.App.Services.ErrorRecorder;


public sealed class OutputErrorRecorder : ErrorRecorder
{
    private readonly string _outputDir = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, $"ErrorRecords/{UID}/Output");
    private readonly string _database;
    private readonly Dictionary<string, int> _logIndex = new();
    private static readonly object Lock = new();

    public OutputErrorRecorder(string database, ILogger logger) : base(logger)
    {
        _database = database;
        Logger = logger;
    }

    /// <summary>
    /// 记录已知表名发生错误的SQL
    /// </summary>
    /// <param name="commandText"></param>
    /// <param name="tableName"></param>
    /// <param name="exception"></param>
    public async Task LogErrorSqlAsync(string commandText, string tableName, Exception exception)
    {
        if (!Directory.Exists(_outputDir))
            Directory.CreateDirectory(_outputDir);
        if (!_logIndex.TryGetValue(tableName, out var idx))
        {
            idx = 0;
            _logIndex.Add(tableName, idx);
        }
        var filePath = Path.Combine(_outputDir, $"{tableName}-{idx}.errlog");
            
        if (File.Exists(filePath) && new FileInfo(filePath).Length > 10 * 1024 * 1024)
        {
            ++idx;
            _logIndex[tableName] = idx;
            filePath = Path.Combine(_outputDir, $"{tableName}-{idx}.errlog");
        }
        var content = $"""
                      /* [{DateTime.Now:yyyy-MM-dd HH:mm:ss}]
                       * Error occurred when export table '{_database}.{tableName}':
                       * {exception.Message}
                       */
                      
                      USE `{_database}`;
                      {commandText}


                      """;
        try
        {
            lock (Lock)
            {
                File.AppendAllText(filePath, content, Encoding.UTF8);
            }
        }
        catch(Exception e)
        {
            Logger.LogError(e, "输出错误日志时发生错误");   
        }
    }

    /// <summary>
    /// 记录发生错误的SQL
    /// </summary>
    /// <param name="commandText"></param>
    /// <param name="exception"></param>
    public async Task LogErrorSqlAsync(string commandText, Exception exception)
    {
        if (!Directory.Exists(_outputDir))
            Directory.CreateDirectory(_outputDir);
        var filePath = Path.Combine(_outputDir, "UnknownTables.errlog");
        var content = $"""
                       /* [{DateTime.Now:yyyy-MM-dd HH:mm:ss}]
                        * Error occurred when export table with unknown table name:
                        * {exception.Message}
                        */
                       {commandText}
                       
                       
                       """;
        await File.AppendAllTextAsync(filePath, content, Encoding.UTF8);
    }

}